<?php

namespace GB\Components\Data\Types;

if(!defined('ABSPATH')) { exit; }

class Giveaway {
	public static function init() {
		self::add_actions();
		self::add_filters();
	}

	private static function add_actions() {
		if(is_admin()) {

		} else {
			add_action('parse_query', [__CLASS__, 'parse_query'], 15);
		}

		add_action('init', [__CLASS__, 'register_type'], 0);
	}

	private static function add_filters() {
		if(is_admin()) {
			add_filter('get_pages', [__CLASS__, 'add_choices_to_homepage_dropdown'], 15, 2);
		} else {
			add_filter('redirect_canonical', [__CLASS__, 'redirect_canonical'], 15, 2);
		}

		add_filter('post_type_link', [__CLASS__, 'post_type_link'], 15, 4);
	}

	#region Front Page Promotions

	public static function add_choices_to_homepage_dropdown($pages, $r) {
		if(isset($r['name']) && 'page_on_front' === $r['name']) {
			$pages = array_merge($pages, get_posts([
				'nopaging'    => true,
				'post_type'   => [GB__DATA_TYPE__GIVEAWAY],
				'post_status' => ['publish'],
				'orderby'     => 'post_title',
				'order'       => 'ASC',
			]));

			uasort($pages, function($a, $b) {
				return strcmp($a->post_title, $b->post_title);
			});
		}

		return $pages;
	}

	public static function post_type_link($post_link, $post, $leavename, $sample) {
		$object_id = intval($post->ID);
		$giveaway  = gb_get_giveaway($object_id);

		if($giveaway && 'page' === get_option('show_on_front') && $object_id === intval(get_option('page_on_front'))) {
			$post_link = home_url('/');
		}

		return $post_link;
	}

	public static function parse_query($wp_query) {
		if ((empty($wp_query->query) || 'true' === $wp_query->query_vars['preview']) && 'page' == get_option('show_on_front') && get_option('page_on_front') && gb_get_giveaway(get_option('page_on_front'))) {
			$wp_query->is_single = true;
			$wp_query->is_page = false;

			$wp_query->set('post_type', GB__DATA_TYPE__GIVEAWAY);
		}
	}

	public static function redirect_canonical($redirect_url, $requested_url) {
		$object_id = intval(get_query_var('p'));

		if($object_id && 'page' === get_option('show_on_front') && $object_id === intval(get_option('page_on_front')) && home_url('/') === $requested_url) {
			$redirect_url = $requested_url;
		}

		return $redirect_url;
	}

	#endregion Front Page Promotions

	#region Registration

	public static function register_type() {
		$labels = [
			'name'                  => __('Promotions'),
			'singular_name'         => __('Giveaway'),
			'add_new'               => __('New Giveaway'),
			'add_new_item'          => __('Add New Promotion'),
			'edit_item'             => __('Edit Promotion'),
			'new_item'              => __('New Giveaway'),
			'view_item'             => __('View Promotion'),
			'view_items'            => __('View Promotions'),
			'search_items'          => __('Search Promotions'),
			'not_found'             => __('No giveaways found.'),
			'not_found_in_trash'    => __('No giveaways found in Trash.'),
			'parent_item_colon'     => null,
			'all_items'             => __('All Promotions'),
			'archives'              => __('Giveaway Archives'),
			'attributes'            => __('Giveaway Attributes'),
			'insert_into_item'      => __('Insert into giveaway'),
			'uploaded_to_this_item' => __('Uploaded to this giveaway'),
			'featured_image'        => __('Featured Image'),
			'set_featured_image'    => __('Set featured image'),
			'remove_featured_image' => __('Remove featured image'),
			'use_featured_image'    => __('Use as featured image'),
			'filter_items_list'     => __('Filter giveaways list'),
			'items_list_navigation' => __('Giveaways list navigation'),
			'items_list'            => __('Giveaways list'),
			'menu_name'             => __('Giveaway Boost'),
		];

		$capabilities = [];

		$supports = [
			'title'
		];

		$rewrite = [
			'slug'       => 'giveaway',
			'with_front' => false,
			'feeds'      => false,
			'pages'      => false,
		];

		register_post_type(GB__DATA_TYPE__GIVEAWAY, [
			'can_export'          => true,
			'capabilities'        => $capabilities,
			'description'         => __('Post type by which giveaways can be managed'),
			'exclude_from_search' => true,
			'has_archive'         => false,
			'hierarchical'        => false,
			'label'               => __('Giveaway'),
			'labels'              => $labels,
			'menu_icon'           => GB__CONF__MENUICON,
			'menu_position'       => GB__CONF__POSITION,
			'public'              => true,
			'publicly_queryable'  => true,
			'rewrite'             => $rewrite,
			'show_in_admin_bar'   => true,
			'show_in_menu'        => true,
			'show_in_nav_menus'   => false,
			'show_ui'             => true,
			'supports'            => $supports,
			'taxonomies'          => [],
		]);
	}

	#endregion Registration

	#region Public API

	public static function get($giveaway) {
		if(empty($giveaway)) { return false; }

		$giveaway = get_post($giveaway);

		if(null === $giveaway || GB__DATA_TYPE__GIVEAWAY !== $giveaway->post_type) { return false; }

		return $giveaway;
	}

	#endregion Public API
}

require_once(path_join(dirname(__FILE__), 'functions/giveaway.functions.php'));

Giveaway::init();
